# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['vpic']

package_data = \
{'': ['*']}

install_requires = \
['desert>=2020.11.18,<2021.0.0', 'requests>=2.25.1,<3.0.0']

setup_kwargs = {
    'name': 'vpic-api',
    'version': '0.7.3',
    'description': 'A client library for the United States National Highway Traffic Safety Administration (NHTSA) Vehicle Product Information Catalog (vPIC) API',
    'long_description': '# vPIC API Client Library\n\n![PyPI](https://img.shields.io/pypi/v/vpic-api)\n![PyPI - Python Version](https://img.shields.io/pypi/pyversions/vpic-api)\n![PyPI - Implementation](https://img.shields.io/pypi/implementation/vpic-api)\n![PyPI - Wheel](https://img.shields.io/pypi/wheel/vpic-api)\n[![MIT License](https://img.shields.io/apm/l/atomic-design-ui.svg?)](https://github.com/tterb/atomic-design-ui/blob/master/LICENSEs)\n[![Tests](https://github.com/davidpeckham/vpic-api/actions/workflows/main.yml/badge.svg)](https://github.com/davidpeckham/vpic-api/actions/workflows/main.yml)\n\nA Python client library for decoding VINs and querying the United States \nNational Highway Traffic Safety Administration (NHTSA) [Vehicle Product \nInformation Catalog Vehicle Listing (vPIC) API](https://vpic.nhtsa.dot.gov/api/).\n\n## NHTSA vPIC API\n\nThe Vehicle Production Information Catalog (vPIC) API is hosted by the U.S. National Highway Transportation Safety Administration (NHTSA). vPIC data is provided by manufacturers who make vehicles for sale in the United States. The vPIC API is available 24/7, is free to use, and does not require registration. See https://vpic.nhtsa.dot.gov/api/home/index/faq for more on the vPIC API.\n\n## Features\n\n- Decode 17-digit Vehicle Identification Numbers (VIN)\n- Decode a partial VIN\n- Discover manufacturers, makes, and models\n- Supports cars, MPVs, trucks, motorcycles, buses, trailers, low speed vehicles and incomplete vehicles sold the U.S.A.\n- Find manufacturer model year VIN guides\n- Supports vehicles made since 1981\n- Comprehensive support for the vPIC API\n- Get results as Python objects (see vpic.TypedClient)\n- Get results as JSON (see vpic.Client)\n\n## Author\n\n- [@davidpeckham](https://github.com/davidpeckham)\n\n## Installation\n\n```bash \n  poetry add vpic-api\n```\n\n## vpic.TypedClient Examples\n\nvpic.TypedClient returns results as Python objects.\n\n### Decode a VIN\n\nGet the characteristics of a vehicle by it\'s 17-digit Vehicle Identification Number (VIN):\n\n```python\nfrom vpic import TypedClient\n\nc = TypedClient()\n\nresult = c.decode_vin("1FTMW1T88MFA00001")\n\nVehicle(\n    abs="",\n    ...\n    body_cab_type="Crew/ Super Crew/ Crew Max",\n    body_class="Pickup",\n    brake_system_desc="",\n    brake_system_type="Hydraulic",\n    ...\n    displacement_cc="3500.0",\n    displacement_ci="213.58310433156",\n    displacement_l="3.5",\n    ...\n    drive_type="4WD/4-Wheel Drive/4x4",\n    ...\n    engine_configuration="V-Shaped",\n    engine_cycles="",\n    engine_cylinders="6",\n    engine_hp="375",\n    engine_hp_to="",\n    engine_kw="279.6375",\n    engine_manufacturer="Ford",\n    engine_model="GTDI",\n    entertainment_system="",\n    error_code="0",\n    error_text="0 - VIN decoded clean. Check Digit (9th position) is correct",\n    ...\n    make="FORD",\n    make_id="460",\n    manufacturer="FORD MOTOR COMPANY, USA",\n    manufacturer_id="976",\n    model="F-150",\n    model_id="1801",\n    model_year="2021",\n    motorcycle_chassis_type="Not Applicable",\n    motorcycle_suspension_type="Not Applicable",\n    ...\n    plant_city="DEARBORN",\n    plant_company_name="",\n    plant_country="UNITED STATES (USA)",\n    plant_state="MICHIGAN",\n    ...\n    series="F-Series",\n    series2="",\n    ...\n    trim="SuperCrew-SSV",\n    ...\n    vin="1FTMW1T88MFA00001",\n    ...\n    vehicle_type="TRUCK ",\n    ...\n)\n```\n\n### Get the Models for a Make and Model Year\n\n```python\nget_models_for_make("TESLA", model_year=2020)\n\n[\n    Model(\n        model_id=1685,\n        model="Model S",\n        make_id=441,\n        make="TESLA",\n        vehicle_type_id=None\n    ),\n    Model(\n        model_id=10199,\n        model="Model X",\n        make_id=441,\n        make="TESLA",\n        vehicle_type_id=None\n    ),\n    Model(\n        model_id=17834,\n        model="Model 3",\n        make_id=441,\n        make="TESLA",\n        vehicle_type_id=None\n    ),\n    Model(\n        model_id=27027,\n        model="Model Y",\n        make_id=441,\n        make="TESLA",\n        vehicle_type_id=None\n        )\n]\n```\n\n## vpic.Client Examples\n\nvpic.Client returns JSON results from the vPIC API.\n\nThis client automatically standardizes variable names where vPIC uses inconsistent naming. Disable this to see the unaltered JSON responses:\n\n```python\nfrom vpic import Client\n\nc = Client(standardize_names=False)\n```\n\n### Decode a VIN\n\nGet the characteristics of a vehicle by it\'s 17-digit Vehicle Identification Number (VIN):\n\n```python\nfrom vpic import Client\n\nc = Client()\n\nresult = c.decode_vin("1FA6P8TD5M5100001", 2021)\n```\n\nHere are a few of the 130+ attributes vPIC returns for the VIN:\n\n```json\n{\n    "Doors": "2",\n    "ErrorCode": "0",\n    "ErrorText": "0 - VIN decoded clean. Check Digit (9th position) is correct",\n    "Make": "FORD",\n    "MakeId": "460",\n    "Manufacturer": "FORD MOTOR COMPANY, USA",\n    "ManufacturerId": "976",\n    "Model": "Mustang",\n    "ModelId": "1781",\n    "ModelYear": "2021",\n    "PlantCity": "FLATROCK",\n    "PlantCountry": "UNITED STATES (USA)",\n    "PlantState": "MICHIGAN",\n    "Series": "I4 Coupe",\n    "VIN": "1FA6P8TD5M5100001",\n    "VehicleType": "PASSENGER CAR",\n}\n```\n\n### Get the Models for a Make and Model Year\n\n```python\nresult = c.get_models_for_make("TESLA", 2021)\n```\n\n```json\n[\n    {\n        "MakeId": 441,\n        "Make": "TESLA",\n        "ModelId": 1685,\n        "Model": "Model S"\n    },\n    {\n        "MakeId": 441,\n        "Make": "TESLA",\n        "ModelId": 10199,\n        "Model": "Model X"\n    },\n    {\n        "MakeId": 441,\n        "Make": "TESLA",\n        "ModelId": 17834,\n        "Model": "Model 3"\n    },\n    {\n        "MakeId": 441,\n        "Make": "TESLA",\n        "ModelId": 27027,\n        "Model": "Model Y"\n    }\n]\n```\n',
    'author': 'David Peckham',
    'author_email': 'dave.peckham@icloud.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/davidpeckham/vpic-api',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
